#pragma once

#include <RobotAPI/interface/armem/query.h>

#include <Ice/Handle.h>

#include <set>
#include <vector>


namespace armarx::armem::server::query_proc::base::detail
{
    // If empty, e.g. when receiving queries from python, we use WM as default.
    // We do it here as (Sl)ice does not support default values for vectors.
    std::set<query::data::QueryTarget> getTargets(const std::vector<query::data::QueryTarget>& _targets);
}
namespace armarx::armem::server::query_proc::base
{

    using QueryTarget = query::data::QueryTarget;


    /**
     * @brief Base class for memory query processors.
     */
    template <QueryTarget queryTarget, class DataT, class ResultT, class QueryT>
    class BaseQueryProcessorBase
    {
    public:

        using QueryPtrT = ::IceInternal::Handle<QueryT>;
        using QuerySeqT = std::vector<QueryPtrT>;


    public:

        virtual ~BaseQueryProcessorBase() = default;

        ResultT process(const QueryT& query, const DataT& data) const
        {
            ResultT result { data.id() };
            if (detail::getTargets(query.targets).count(queryTarget))
            {
                this->process(result, query, data);
            }
            return result;
        }

        ResultT process(const QueryPtrT& query, const DataT& data) const
        {
            return this->process(*query, *data);
        }

        ResultT process(const QuerySeqT& queries, const DataT& data) const
        {
            ResultT result { data.id() };
            this->process(result, queries, data);
            return result;
        }

        void process(ResultT& result, const QuerySeqT& queries, const DataT& data) const
        {
            if (queries.empty())
            {
                return;
            }

            for (const auto& query : queries)
            {
                if (detail::getTargets(query->targets).count(queryTarget))
                {
                    this->process(result, *query, data);
                }
            }
        }


        /**
         * @brief Process the query and populate `result`.
         *
         * @param result The result container.
         * @param query The query.
         * @param data The source container.
         */
        virtual void process(ResultT& result, const QueryT& query, const DataT& data) const = 0;

    };


}
