#include "QueryWidget.h"

#include <limits>

#include <QCheckBox>
#include <QGroupBox>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QSpinBox>
#include <QTabWidget>
#include <QVBoxLayout>
#include <QWidget>

#include <ArmarXCore/core/exceptions/local/ExpressionException.h>


namespace armarx::armem::gui
{

    QueryWidget::QueryWidget()
    {
        auto* vlayout = new QVBoxLayout();

        _dataCheckBox = new QCheckBox("Get Data");
        _dataCheckBox->setChecked(true);
        vlayout->addWidget(_dataCheckBox);

        auto* recDepthHLayout = new QHBoxLayout();
        vlayout->addLayout(recDepthHLayout);

        _recursionDepthSpinner = new QSpinBox(); // NOLINT
        _recursionDepthSpinner->setRange(-1, std::numeric_limits<int>::max());
        _recursionDepthSpinner->setValue(0);
        _recursionDepthSpinner->setEnabled(_dataCheckBox->isChecked());
        _recursionDepthSpinner->setSpecialValueText(QString("Unlimited"));
        recDepthHLayout->addWidget(_recursionDepthSpinner);

        _recursionDepthLabel = new QLabel("Link resolution depth");
        recDepthHLayout->addWidget(_recursionDepthLabel);
        recDepthHLayout->addStretch();

        _storeInLTMButton = new QPushButton("Store query result in LTM");
        vlayout->addWidget(_storeInLTMButton);

        // Public connections.
        connect(_storeInLTMButton, &QPushButton::pressed, this, &This::storeInLTM);

        // Private connections.
        connect(
            _dataCheckBox, &QCheckBox::stateChanged, this, &This::setRecursionDepthSpinnerEnabled);

        setLayout(vlayout);
    }


    armem::DataMode QueryWidget::dataMode() const
    {
        return _dataCheckBox->isChecked()
               ? armem::DataMode::WithData
               : armem::DataMode::NoData;
    }

    int QueryWidget::queryLinkRecursionDepth() const
    {
        return _recursionDepthSpinner->value();
    }

    void
    QueryWidget::setRecursionDepthSpinnerEnabled(int state)
    {
        switch (state)
        {
            case Qt::Checked:
                _recursionDepthSpinner->setEnabled(true);
                break;
            case Qt::Unchecked:
            default:
                _recursionDepthSpinner->setEnabled(false);
                break;
        }
    }
}
